makePipelineFile  <- function(fileName,shiftSize=5000,segmentSize=10000) {
sourcefile <- file.path(system.file(package="hapFabia"),"pipeline/pipeline.R")

source1 <- readLines(sourcefile)

write(paste("#####define segments, overlap, filename #######",sep=""),file="pipeline.R",ncolumns=100)
write(paste("shiftSize <- ",shiftSize,sep=""),file="pipeline.R",append=TRUE,ncolumns=100)
write(paste("segmentSize <- ",segmentSize,sep=""),file="pipeline.R",append=TRUE,ncolumns=100)
write(paste("fileName <- \'",fileName,"\' # without type",sep=""),file="pipeline.R",append=TRUE,ncolumns=100)

write.table(source1,file="pipeline.R",append=TRUE,quote = FALSE,row.names = FALSE,col.names = FALSE)

}

hapFabiaVersion <- function() {
  version <- packageDescription("hapFabia",fields="Version")
  cat('\nhapFabia Package Version ', version, '\n')
  cat('\nCopyright (c) 2012 by Sepp Hochreiter\n\n')
}

topLZ <- function(res,n=1,LZ="L",indices=TRUE,p=NULL,w=NULL) {

if (missing(res)) {
    stop("Fabia result 'res' is missing. Stopped.")
}

if ((is.null(p))&&(is.null(w))) {

    stop("Either 'p' or 'w' must be given. Stopped.")

}

if (LZ=="Z") {

    if (!is.null(p)) {
        if (indices) {
            return(which(res@Z[n,]>quantile(res@Z[n,],p)))
        } else {
          return(res@Z[n,which(res@Z[n,]>quantile(res@Z[n,],p))])
        }
    } else {
        if (indices) {
            return(which(res@Z[n,]>w))
        } else {
            return(res@Z[n,which(res@Z[n,]>w)])
        }

    }

} else {

    if (!is.null(p)) {
        if (indices) {
            return(which(res@L[,n]>quantile(res@L[,n],p)))
        } else {
          return(res@L[which(res@L[,n]>quantile(res@L[,n],p)),n])
        }
    } else {
        if (indices) {
            return(which(res@L[,n]>w))
        } else {
            return(res@L[which(res@L[,n]>w),n])
        }

    }
}


}

plotL <- function(res,n=1,p=NULL,w=NULL,type="points",intervv=500,off=0,t="p",cex=1) {

if (missing(res)) {
    stop("Fabia result 'res' is missing. Stopped.")
}

if ((is.null(p))&&(is.null(w))) {

    stop("Either 'p' or 'w' must be given. Stopped.")

}

if (!is.null(p)) {

    if (type=="points") {
        a <- topLZ(res=res,n=n,p=p,LZ="L",indices=TRUE)
        b <- topLZ(res=res,n=n,p=p,LZ="L",indices=FALSE)
        plot(a,b,xlim=c(0,dim(res@L)[1]),col="blue",type=t,cex=cex,main=paste("bicluster ",n,sep=""),ylab="counts",xlab="SNV number")

    } else {
        if (type=="histogram") {
            off <- off%%intervv
            bb <- seq(1-off,length(res@L[,n]),intervv)
            lb <- length(bb)
            bb <- c(bb,(bb[lb]+intervv))
            a <- topLZ(res=res,n=n,p=p,LZ="L",indices=TRUE)
            aa <- hist(a,breaks=bb,plot = FALSE)
            plot(aa,freq = TRUE,xlim=c(0,dim(res@L)[1]),col="blue",main=paste("bicluster ",n,sep=""),ylab="counts",xlab="SNV number")

        } else {
            a <- topLZ(res=res,n=n,p=p,LZ="L",indices=TRUE)
            b <- topLZ(res=res,n=n,p=p,LZ="L",indices=FALSE)
            smoothScatter(a,b,xlim=c(0,dim(res@L)[1]),main=paste("bicluster ",n,sep=""),ylab="counts",xlab="SNV number")

        }

    }


} else {

    if (type=="points") {
        a <- topLZ(res=res,n=n,w=w,LZ="L",indices=TRUE)
        b <- topLZ(res=res,n=n,w=w,LZ="L",indices=FALSE)
        plot(a,b,xlim=c(0,dim(res@L)[1]),col="blue",type=t,cex=cex,main=paste("bicluster ",n,sep=""),ylab="counts",xlab="SNV number")

    } else {
        if (type=="histogram") {
            off <- off%%intervv
            bb <- seq(1-off,length(res@L[,n]),intervv)
            lb <- length(bb)
            bb <- c(bb,(bb[lb]+intervv))
            a <- topLZ(res=res,n=n,w=w,LZ="L",indices=TRUE)
            aa <- hist(a,breaks=bb,plot = FALSE)
            plot(aa,freq = TRUE,xlim=c(0,dim(res@L)[1]),col="blue",main=paste("bicluster ",n,sep=""),ylab="counts",xlab="SNV number")

        } else {
            a <- topLZ(res=res,n=n,w=w,LZ="L",indices=TRUE)
            b <- topLZ(res=res,n=n,w=w,LZ="L",indices=FALSE)
            smoothScatter(a,b,xlim=c(0,dim(res@L)[1]),main=paste("bicluster ",n,sep=""),ylab="counts",xlab="SNV number")

        }

    }


}


}

histL <- function(res,n=1,p=NULL,w=NULL,intervv=500,off=0) {

    if (missing(res)) {
        stop("Fabia result 'res' is missing. Stopped.")
    }

    if ((is.null(p))&&(is.null(w))) {

        stop("Either 'p' or 'w' must be given. Stopped.")

    }

    off <- off%%intervv
    bb <- seq(1-off,length(res@L[,n]),intervv)
    lb <- length(bb)
    bb <- c(bb,(bb[lb]+intervv))
    if (!is.null(p)) {
        a <- topLZ(res=res,n=n,p=p,LZ="L",indices=TRUE)
    } else {
        a <- topLZ(res=res,n=n,w=w,LZ="L",indices=TRUE)
    }
    rhist <- hist(a,breaks=bb,plot = FALSE)
    return(rhist)
}



extractIBD <- function(rrnn,p=0.9,inte=500,thres=11,off=0) {

    if (missing(rrnn)) {
        stop("Vector 'rrnn' for constructing histogram is missing. Stopped.")
    }

    off <- off%%inte
    bb <- seq(1-off,length(rrnn),inte)
    lb <- length(bb)
    bb <- c(bb,(bb[lb]+inte))
    a2 <- which(rrnn>quantile(rrnn,p))
    hh1 <- hist(a2,breaks=bb,plot = FALSE)

    a1 <- which(hh1$counts>thres)
    la <- length(a1)

    if (la>0) {

      b1 <- round(hh1$mid[a1])


      b2 <- rep(inte,la)

      b3 <- list()
      b4 <- rep(0,la)


      for (i in 1:la) {

        range <- (b1[i]-(inte%/%2)):(b1[i]+(inte%/%2))
        a3 <- intersect(range,a2)
        b3[[i]] <- sort(a3)
        b4[i] <- length(a3)
      }
      return(list(m=b1,l=b2,pos=b3,len=b4))
    } else {
      return(list(m=NULL,l=NULL,pos=NULL,len=NULL))
    }
}



snpPlot <- function(x,range=NULL, yLabels=NULL, zlim=NULL,title=NULL,colRamp=12){


     if (missing(x)) {
            stop("Data matrix x missing. Stopped.")
        }

        if (!is.matrix(x)) {
            x <- as.matrix(x)
        }

        if (!is.numeric(x)) {
            stop("Data matrix x must be numeric. Stopped.")
        }



     min <- min(x)
     max <- max(x)

      if( !is.null(zlim) ){
       min <- zlim[1]
       max <- zlim[2]
    }

    if( !is.null(yLabels) ){
       yLabels <- c(yLabels)
    } else {
       yLabels <- c(1:nrow(x))
   }

    if( !is.null(range) ){
      xLabels <- c(range)
    } else {
      xLabels <- c(1:ncol(x))
    }

     if( is.null(title) ){
     title <- c()
    }

 # Red and green range from 0 to 1 while Blue ranges from 1 to 0

         seq00 <- seq(0,0,length=256)
         seq11 <- seq(1,1,length=256)

     if (colRamp<9) {
         seq01 <- seq(0,1,length=256)
         seq10 <- seq(1,0,length=256)
         seq051 <- seq(0.5,1,length=256)
         seq005 <- seq(0,0.5,length=256)

         dev1 <- cbind(seq01,seq01,seq10)
         dev2 <- cbind(seq01,seq051,seq10)
         dev3 <- cbind(seq01,seq005,seq10)
         blue2green <- cbind(seq00,seq01,seq10) # blue2green
         green2red <- cbind(seq01,seq10,seq00) # green2red
         blue2yellow <- cbind(seq01,seq01,seq10) # blue2yellow
         cyan2yellow <- cbind(seq01,seq11,seq10) # cyan2yellow
         magenta2green <- cbind(seq10,seq01,seq10) # magenta2green
     } else {
    if (colRamp==9) {



        seqP1 <- c(0,3,5,8,11,13,16,19,21,24,27,29,32,35,37,40,42,45,48,50,53,56,58,61,64,66,69,72,74,77,80,82,85,88,90,93,96,98,101,104,106,109,112,114,117,120,122,125,128,130,133,135,138,141,143,146,149,151,154,157,159,162,165,167,170,173,175,178,181,183,186,189,191,194,197,199,202,205,207,210,212,215,218,220,223,226,228,231,234,236,239,242,244,247,250,252) # 96 steps

        seqP2 <- rep(255,64)
        seqP3 <- seqP1[96:1]
        seqP4 <- rep(0,64)
        seqP5 <- seqP1[1:32]
        seqP6 <- seqP3[1:32]


        seqM1 <- c(seqP1,seqP2,seqP3)/255
        seqM2 <- c(seqP4,seqP1,seqP2,seqP6)/255
        seqM3 <- seqM1[256:1]
        matlab <- cbind(seqM1,seqM2,seqM3)
    } else {
        seq0055 <- seq(0,55/256,length=256)
        seqQ1 <- c(0,2,5,7,10,12,15,18,20,22,25,28,30,32,35,38,40,42,45,48,50,52,55,58,60,62,65,68,70,72,75,77,80,83,85,88,90,92,95,97,100,103,105,108,110,112,115,117,120,123,125,128,130,133,135,137,140,143,145,147,150,152,155,158,160,162,165,167,170,173,175,177,180,183,185,187,190,192,195,198,200,202,205,207,210,213,215,217,220,223,225,227,230,232,235,238,240,242,245,247,250,253) # 102 steps

        seqQ2 <- rep(255,52)
        seqQ3 <- seqQ1[102:77]
        seqQ4 <- rep(0,76)

        seqQ5 <- rep(0,26)
        seqQ6 <- (1:50)*5
        seqQ7 <- rep(255,104)
        seqQ8 <- seqQ6[50:1]

        seqQ9 <- seqQ3[26:1]
        seqQ10 <- rep(255,52)
        seqQ11 <- seqQ1[102:1]
        seqQ12 <- rep(0,76)

        seqL1 <- c(seqQ4,seqQ1,seqQ2,seqQ3)/255
        seqL2 <- c(seqQ5,seqQ6,seqQ7,seqQ8,seqQ5)/255
        seqL3 <- c(seqQ9,seqQ10,seqQ11,seqQ12)/255


        blue2green2red <- cbind(seqL1,seqL2,seqL3) # blue2green2red
        sepp1 <- cbind(seq11,seq11,seq11) - blue2green2red
        sepp2 <- sepp1
        sepp3 <- cbind(seq00,seq11,seq00) - blue2green2red
        sepp4 <- cbind(seq00,seq0055,seq00) - blue2green2red
        sepp5 <- cbind(seq11,seq00,seq11) - blue2green2red
        sepp6 <- sepp5

    }
}

    ColorRamp <- switch(colRamp,
    rgb(dev1),
    rgb(dev2),
    rgb(dev3),
    rgb(blue2green),
    rgb(green2red),
    rgb(blue2yellow),
    rgb(cyan2yellow),
    rgb(magenta2green),
    rgb(matlab),
    rgb(blue2green2red),
    rgb(blue2green2red),
    rgb(sepp1),
    rgb(sepp2),
    rgb(sepp3),
    rgb(sepp4),
    rgb(sepp5),
    rgb(sepp6),
      )

 # Reverse Y axis
 reverse <- nrow(x) : 1
 yLabels <- yLabels[reverse]
 x <- x[reverse,]

 # Data Map
# par(mar = c(3,5,2.5,2))
# par(mar = c(3,5,2.5,1))
# Original: par(mar=c(5.1,4.1,4.1,2.1))
 marO <- par("mar")
 par(mar = c(2.5,6,3,1))
 image(1:ncol(x), 1:nrow(x), t(x), col=ColorRamp, xlab="",
 ylab="", axes=FALSE, zlim=c(min,max))
 if( !is.null(title) ){
    title(main=title)
 }

xt <- c(1,axTicks(1))

lxt <- length(xt)

if (abs(xt[lxt]-ncol(x))>5) {
    xt <- c(xt,ncol(x))
}

axis(BELOW<-1, at=xt, labels=prettyNum(xLabels[xt], big.mark = ","), cex.axis=0.7)

axis(LEFT <-2, at=1:length(yLabels), labels=yLabels, las= HORIZONTAL<-1,cex.axis=0.7)

 par(mar = marO)

 layout(1)
}





plotRangeL <- function(Lout,ibb,physPos=NULL,colRamp=12,val=c(0.0,2.0,1.0),chrom=1,count=0,labelsNA=NULL,prange=NULL,labelsNA1 = c("model L") ) {

    if (missing(Lout)) {
        stop("Sample SNVs 'Lout' are missing. Stopped.")
    }

    if (missing(ibb)) {
        stop("tagSNVs 'ibb' are missing. Stopped.")
    }



n <- dim(Lout)[2]
n1 <- length(ibb)
doMis <- FALSE
if (n1>4) {
    n1 <- n1-1
    doMis <- TRUE
  }

if ((!is.null(prange))&&(!is.null(physPos))) {

if (length(prange)==2) {
ma <- length(ibb[[1]])

a1 <- which(physPos>=prange[1])
a2 <- which(physPos<=prange[2])

aa <- intersect(a1,a2)

if (length(aa)>1) {

  ibb <- ibb[[1]][aa]
  physPos <- physPos[aa]

}

}

}

if (is.null(physPos)) {
    physPos <- ibb*100
}


ma <- length(ibb[[1]])


range <- ibb[[1]][1]:ibb[[1]][ma]

lr <- ibb[[1]][ma]-ibb[[1]][1]+1

physRanget <- physPos[1]:physPos[ma]

lt <- length(physRanget)

ltr <- lt%/%(lr-1)

ltt <- c(1,(1:(lr-2))*ltr,lt)

physRange <- physRanget[ltt]

physLength <- round(lt/1000)

physPosM <- (physPos[ma] + physPos[1])%/%2


mat <- matrix(val[1],(n+n1+1),lr)
mat[(n+1),] <- val[2]

for (iB in 1:n1) {

  ibb_pos <- match(ibb[[iB]],range)

  mat[(n+1+iB),ibb_pos] <- val[3]
}
if (doMis) {
   ibb_pos <- match(ibb[[n1+1]],range)
   mat[(n+1+3),ibb_pos] <- 0.35*val[3]
}

ibb_pos <- match(ibb[[1]],range)

for (j in 1:n) {
    lla <- which(Lout[range,j]>=0.1)
    llb <- setdiff(1:lr,lla)
    llc <- intersect(ibb_pos,lla)
    lla <- setdiff(lla,llc)

    mat[j,lla] <- val[2]
    mat[j,llb] <- val[1]
    mat[j,llc] <- val[3]

}

if (is.null(labelsNA)) {
    labelsNA <- as.character(1:n)
}

if (length(labelsNA1)!=n1) {
  if (n1>1) {
  ade <- rep("",(n1-1))
  labelsNA1 = c("model L",ade)
} else {
  labelsNA1 = c("model L")
}
}

labelsNA <- c(labelsNA,"",labelsNA1)

if (count<=0) {
snpPlot(mat,range=physRange,yLabels=labelsNA, title=paste("chr: ",chrom,"  ||  pos: ",prettyNum(physPosM, big.mark = ","),"  ||  length: ",physLength,"kbp  ||  #SNPs: ",ma,"  ||  #Samples: ",n,sep=""),colRamp=colRamp)
} else {
snpPlot(mat,range=physRange,yLabels=labelsNA, title=paste("count: ",count,"  ||  chr: ",chrom,"  ||  pos: ",prettyNum(physPosM, big.mark = ","),"  ||  length: ",physLength,"kbp  ||  #SNPs: ",ma,"  ||  #Samples: ",n,sep=""),colRamp=colRamp)
}


}


plotOneIBD <- function(IBD,filename,ibdC=1) {

    if (missing(IBD)) {
        stop("List of haplotypes 'IBD' is missing. Stopped.")
    }

    if (missing(filename)) {
        stop("File name 'filename' for loading sample is missing. Stopped.")
    }



if (length(IBD)>=ibdC) {

samp <- IBD[[ibdC]]$noSamples
ibb <- IBD[[ibdC]]$noSnps
ibb <- as.integer(sort.int(as.integer(unique(ibb))))
snpPositions <- IBD[[ibdC]]$snpPositions

labels_ALL <- IBD[[ibdC]]$labelSamples

Lout <- readSamplesSpfabia(X=filename,samples=samp,lowerB=0,upperB=1000.0)


ibbL <- list(ibb)
labelsK <- c("model L")

# here other annotations can be included
# ibbL <- list(ibb,ibbAnnot1,ibbAnnot2)
# labelsK <- c("model L","Annot1",Annot2")

plotRangeL(Lout=Lout,ibb=ibbL,physPos=snpPositions,colRamp=12,val=c(0.0,2.0,1.0),chrom=1,count=0,labelsNA=labels_ALL,labelsNA1=labelsK)

} else {
    message("No plot because haplotype ",ibdC," does not exist!")

}

}





plotIBDs <- function(IBD,filename) {


    if (missing(IBD)) {
        stop("List of haplotypes 'IBD' is missing. Stopped.")
    }

    if (missing(filename)) {
        stop("File name 'filename' for loading sample is missing. Stopped.")
    }

devAskNewPage(ask = TRUE)

for (i in 1:length(IBD)) {


samp <- IBD[[i]]$noSamples
ibb <- IBD[[i]]$noSnps
ibb <- as.integer(sort.int(as.integer(unique(ibb))))
snpPositions <- IBD[[i]]$snpPositions

labels_ALL <- IBD[[i]]$labelSamples

Lout <- readSamplesSpfabia(X=filename,samples=samp,lowerB=0,upperB=1000.0)


plotRangeL(Lout=Lout,ibb=list(ibb),physPos=snpPositions,colRamp=12,val=c(0.0,2.0,1.0),chrom=1,count=i,labelsNA=labels_ALL)

}


devAskNewPage(ask = FALSE)

}




getIBDs <- function(res,sPF,annot=NULL,chrom=1,labelsA=NULL,ps=0.9,psZ=0.8,inteA=500,thresA=11,minSNPs=8,off=0,procMinSamps=0.1,thresPrune=1e-3) {

    if (missing(res)) {
        stop("Fabia result 'res' is missing. Stopped.")
    }

    if (missing(sPF)) {
        stop("Samples per features 'sPF' is missing. Stopped.")
    }

if (is.null(labelsA)) {
  nnL <- length(res@Z[1,])
  labelsA <- cbind(as.character(1:nnL),as.character(1:nnL),as.character(1:nnL),as.character(1:nnL))
}


if (is.null(annot)) {

  snps <- length(res@L[,1])

  dummyL <- 1:snps
  dummyC <- rep(0,snps)
  dummyN <- rep("N",snps)
  dummy <- as.character(dummyL)

  annot <- list()

  annot[[1]] <- dummyL
  annot[[2]] <- 100*dummyL
  annot[[3]] <- dummy
  annot[[4]] <- dummyN
  annot[[5]] <- dummyN
  annot[[6]] <- dummy
  annot[[7]] <- dummy
  annot[[8]] <- dummy
  annot[[9]] <- dummy
  annot[[10]] <- as.double(dummyC)
  annot[[11]] <- dummyC

}

ibd_res <- list()
ibd_res_idx <- 1

max_n <- ncol(res@L)

for (n in 1:max_n) {

if (max(res@L[,n])>0.000001) {


ib <- extractIBD(res@L[,n],p=ps,inte=inteA,thres=thresA,off=off)

topZ <- which(res@Z[n,]>quantile(res@Z[n,],psZ))


if ((!is.null(ib$len))&&(length(ib$len)>0)) {

for (i in 1:length(ib$len)) {


ibb <- as.vector(unlist(ib$pos[[i]]))
#ma <- length(ibb)

il <- length(ibb)


aa <- c()
nu <- list()
for (j in 1:il) {
  nu[[j]] <- sPF$sL[[ibb[j]]]
  aa <- c(aa,nu[[j]])
}

aa <- unique(aa)

aa <- intersect(aa,topZ)

lq <- length(aa)

if (lq>1) {

aa <- sort(aa)



mat <- matrix(0,lq,il)

for (j in 1:il) {
  nv <- which(match(aa,nu[[j]])>0)
  mat[nv,j] <- rep(1,length(nv))
}



select1 <- which(rowSums(mat)>minSNPs)

lq1 <- length(select1)


if (lq1>1) {

noSample1 <- aa[select1]



mat1 <- mat[select1,]

select2 <- which(colSums(mat1)>1)

lq2 <- length(select2)

if (lq2>1) {

mat1 <- mat1[,select2]


noSnp1 <- ibb[select2]



matB <- mat1
ig=1
ie=1
while (ie == 1) {


    nnc <- ncol(matB)
    subclS <- 1:nnc
    lq2A <- nnc


    nnl <- nrow(matB)
    subclM <- 1:nnl
    lq1A <- nnl



    lq2A_old <- lq2A
    lq1A_old <- lq1A


    done=0

    if ((lq1A<2)||(lq2A<2)) {
      lq1A <- 1
      done <- 1
    }

    while (done==0) {




      matA <- matB[subclM,subclS]

      cs1 <- rowSums(matA)

      m2 <- which.max(cs1)

      v1 <- matA%*%matA[m2,]

      v1A <- v1
      v1A[m2] <- 0

      m3 <- which.max(v1A)

      v2 <-  matA%*%matA[m3,]

      subclM1A <- which(v1>minSNPs)
      subclM1B <- which(v2>minSNPs)

      subclM1 <- intersect(subclM1A,subclM1B)

      lq1A <- length(subclM1)



     if (lq1A>1)  {



      cs2 <- colSums(matA[subclM1,])

      subclS1 <- which(cs2>max(1,(lq1A%/%(1/procMinSamps))))

      lq2A <- length(subclS1)



      subclM <- subclM[subclM1]
      subclS <- subclS[subclS1]

      subclM2 <- subclM[c(m2,m3)]




      if ((lq1A_old==lq1A)&&(lq2A_old==lq2A)) {
        done <- 1
      } else {
        lq2A_old <-  lq2A
        lq1A_old <-  lq1A
      }

       if ((lq1A<2)||(lq2A<2)) {
        lq1A <- 1
        done <- 1
      }

    } else {
       done <- 1
    }

    }


    if (lq1A>1) {


      a0 <- noSnp1[subclS]
      a1 <- diff(a0)
      m <- 1/(max(2,median(a1)))
      k0 <- length(a1)
      k <- k0
      wegS <- 0
      wegE <- 0
      thresPruneN <- 1.0 - thresPrune
      if (k>2) {
        doPrune <- TRUE
      } else {
        doPrune <- FALSE
      }

      while (doPrune) {

        doPrune <- FALSE

        if ((k>2)&&(pexp(a1[1],m) > thresPruneN)) {
          a1 <- a1[-1]
          k <- k - 1
          m <- 1/(max(2,median(a1)))
          wegS <- wegS+1
          doPrune <- TRUE
        } else {
          if ((k>2)&&(pexp(a1[2],m) > thresPruneN)) {
            a1 <- a1[c(-2,-1)]
            k <- k - 2
            m <- 1/(max(2,median(a1)))
            wegS <- wegS+2
            doPrune <- TRUE
          }

        }

        if ((k>2)&&(pexp(a1[k],m) > thresPruneN)) {
          a1 <- a1[-k]
          k <- k - 1
          m <- 1/(max(2,median(a1)))
          wegE <- wegE+1
          doPrune <- TRUE
        } else {
          if ((k>2)&&(pexp(a1[k-1],m) > thresPruneN)) {
            a1 <- a1[c(-k,(-k+1))]
            k <- k - 2
            m <- 1/(max(2,median(a1)))
            wegE <- wegE+2
            doPrune <- TRUE
          }

        }

      }

      subclS <- subclS[(1+wegS):(k0-wegE)]
      lq2A <- length(subclS)



      if (lq2A>=minSNPs) {


      matA <- matB[subclM,subclS]


      noSample <- noSample1[subclM]
      noSample2 <- noSample1[subclM2]

      labelsEUA <- labelsA[noSample,2]
      labelsNAA <- labelsA[noSample,1]
      labelsTechA <- labelsA[noSample,4]
      labels_ALLA <- paste(labelsNAA,labelsEUA,sep="_")

      labelsNA2 <- labelsA[noSample2,1]

      noSnp <- noSnp1[subclS]


      physRangeA <- annot[[2]][noSnp]
      ibdLength <-  physRangeA[lq2A]-physRangeA[1]
      ibdPos <-  (physRangeA[lq2A]+physRangeA[1])%/%2
      allelesA <- paste(annot[[4]][noSnp],":",annot[[5]][noSnp],sep="")
      snpNamesA <- annot[[3]][noSnp]
      snpFreqA <- annot[[10]][noSnp]
      nnn <- length(labelsA[,1])
      snpGroupFreqA <- sPF$nsL[noSnp]/nnn
      snpChangeA <- annot[[11]][noSnp]

      snpsPerSample <- rowSums(matA)
      samplePerSnp <- colSums(matA)


        res_t <- list(number=ibd_res_idx,bicluster=n,chrom=chrom,ibdPos=ibdPos,ibdLength=ibdLength,numberSamples=lq1A,numberSnps=lq2A,noSamples=noSample,noSnps=noSnp,countrySamples=labelsEUA,idSamples=labelsNAA,labelSamples=labels_ALLA,techSamples=labelsTechA,maxSamples=labelsNA2,snpPositions=physRangeA,snpAlleles=allelesA,snpNames=snpNamesA,snpFreq=snpFreqA,snpGroupFreq=snpGroupFreqA,snpChange=snpChangeA,snpsPerSample=snpsPerSample,samplePerSnp=samplePerSnp)

        ibd_res[[ibd_res_idx]] <- res_t
        ibd_res_idx <- ibd_res_idx +1

        ig <- ig + 1
        if ((lq1A<(nnl-1))&&(lq2A<(nnc-1))) {

            matB <-  matB[-subclM,-subclS]

            noSample1 <- noSample1[-subclM]


            noSnp1 <- noSnp1[-subclS]

       } else {
            ie=100
        }

        } else {
            ie=100
        }


    } else {
        ie=100
    }




  }

}

}

}
}
}
}

}

return(ibd_res)

}



ibd2excel <- function(resIBD,filename) {

    if (missing(resIBD)) {
        stop("List of haplotypes 'resIBD' is missing. Stopped.")
    }
    if (missing(filename)) {
        stop("File name 'filename' of the EXCEL file is missing. Stopped.")
    }

libd <- length(resIBD)
if (libd>0)  {
lentry <- length(resIBD[[1]])
if (lentry<25)  {
out <- c("numb , ","bicluster , ","chrom , ","ibdPos , ","ibdLength , ","numberSamples , ","numberSnps , ","SampleNumber , ","SnpNumber , ","countrySamples , ","idSamples , ","labelSamples , ","techSamples , ","maxSamples , ","snpPositions , ","snpAlleles , ","snpNames , ","snpFreq , ","snpGroupFreq , ","snpChange , ","snpsPerSample , ","samplePerSnp")
} else  {
out <- c("numb , ","bicluster , ","chrom , ","ibdPos , ","ibdLength , ","numberSamples , ","numberSnps , ","SampleNumber , ","SnpNumber , ","countrySamples , ","idSamples , ","labelSamples , ","techSamples , ","maxSamples , ","snpPositions , ","snpAlleles , ","snpNames , ","snpFreq , ","snpGroupFreq , ","snpChange , ","snpsPerSample , ","samplePerSnp , ","exonicPosition , ","exonicAnnotation , ","exonicCount , ","variantPosition , ","variantAnnotation , ","variantCount , ","TranscriptionFactorPosition , ","TranscriptionFactorAnnotation")
}


write.table(t(out),file=filename,sep=" ",quote = FALSE,row.names = FALSE,col.names = FALSE)
for (i in 1:libd) {

    out <- c(resIBD[[i]][[1]])
    for (j in 2:lentry) {

      out <- c(out,",",resIBD[[i]][[j]])

    }

    write.table(t(out),file=filename,sep=" ",quote = FALSE,row.names = FALSE,col.names = FALSE,append=TRUE)

}
} else {
out <- c("numb , ","bicluster , ","chrom , ","ibdPos , ","ibdLength , ","numberSamples , ","numberSnps , ","SampleNumber , ","SnpNumber , ","countrySamples , ","idSamples , ","labelSamples , ","techSamples , ","maxSamples , ","snpPositions , ","snpAlleles , ","snpNames , ","snpFreq , ","snpGroupFreq , ","snpChange , ","snpsPerSample , ","samplePerSnp")

write.table(t(out),file=filename,sep=" ",quote = FALSE,row.names = FALSE,col.names = FALSE)

}

}


sim <- function(x,y,simv="minD",minInter=2) {

    if ((missing(x))||(missing(y))) {
        stop("'x' or 'y' is missing. Stopped.")
    }

  intervv1 <- length(intersect(x,y))

  if (intervv1>=minInter) {
     erg <- switch(simv,
       jaccard = intervv1/length(union(x,y)),
       dice = 2*intervv1/(length(x)+length(y)),
       minD = intervv1/ min(length(x),length(y)),
       maxD = intervv1/ max(length(x),length(y))
     )
   } else {
     erg <- 0
   }

  return(erg)

}


compareIBDres <- function(resIBD1,resIBD2=NULL,simv="minD",pSnps=NULL,pSamp=NULL,minSnps=6,minSamp=2) {

    if (missing(resIBD1)) {
        stop("List of haplotypes 'resIBD1' is missing. Stopped.")
    }

    if (!is.null(resIBD2)) {
     snps1 <- sapply(resIBD1,function(x) {x[[9]]} , simplify=FALSE)
     snps2 <- sapply(resIBD2,function(x) {x[[9]]}  , simplify=FALSE)
     samp1 <- sapply(resIBD1,function(x) {x[[8]]}  , simplify=FALSE)
     samp2 <- sapply(resIBD2,function(x) {x[[8]]}  , simplify=FALSE)

     snps <- c(snps1,snps2)
     samp <- c(samp1,samp2)
  } else {

     snps <- sapply(resIBD1,function(x) {x[[9]]} , simplify=FALSE )
     samp <- sapply(resIBD1,function(x) {x[[8]]}  , simplify=FALSE)
  }

  l <- length(snps)

  if (l>1) {

  snpsSim <- sapply(snps,function(x) { sapply(snps, function(y) {sim(x,y,simv,minInter=minSnps)})} )

  if (!is.null(pSnps)) {
    snpsSim <- snpsSim^pSnps
  }

  sampSim <- sapply(samp,function(x) { sapply(samp, function(y) {sim(x,y,simv,minInter=minSamp)})} )

  if (!is.null(pSamp)) {
      sampSim <- sampSim^pSamp
  }
  oneM <- matrix(1,nrow=l,ncol=l)

  dist <- as.dist(oneM - snpsSim*sampSim)


  clust <- hclust(dist)

  } else {

      clust=NULL
  }

  return(clust)

}



mergeIBDres <- function(resIBD1,resIBD2=NULL,comp,cut=0.8) {

    if (missing(resIBD1)) {
        stop("List of haplotypes 'resIBD1' is missing. Stopped.")
    }

    if (missing(comp)) {
        stop("Result 'comp' of hclust is missing. Stopped.")
    }

  clustIBD <- cutree(comp,h=cut)
  l <- length(resIBD1)
  if (!is.null(resIBD2)) {
      l2 <- length(resIBD2)
  } else {
      l2 <- 0
  }

  tabClust <- table(clustIBD)
  cl <- length(tabClust)

  resIBDmerge <- list()
  for (i in 1:cl) {


    tr1 <- which(clustIBD==i)
    ltr1 <- length(tr1)

    vv <- list()

    p2 <- tr1[1]

    if (p2<=l) {
      for (j in 1:22) {
        vv[[j]] <- resIBD1[[p2]][[j]]
      }
    } else  {
       if ((l2>0)&&(p2<=l+l2)) {
          p2 <- p2-l
          for (j in 1:22) {
            vv[[j]] <- resIBD2[[p2]][[j]]
          }
        }
    }

    if (ltr1>1) {
    for (j in 2:ltr1) {


     p2 <- tr1[j]
      if (p2<=l) {

        ri1 <- resIBD1[[p2]][[8]]
        ri2 <- resIBD1[[p2]][[9]]

        ma1 <- match(ri1,vv[[8]])
        ma2 <- match(ri2,vv[[9]])
        na1 <- which(ma1>0)
        na2 <- which(ma2>0)

        if (length(na1)>0) {
          ma1 <- ma1[na1]
          vv[[21]][ma1] <- pmax(vv[[21]][ma1],resIBD1[[p2]][[21]][na1])
         }

        if (length(na2)>0) {
          ma2 <- ma2[na2]
          vv[[22]][ma2] <- pmax(vv[[22]][ma2],resIBD1[[p2]][[22]][na2])
        }


        ab1 <- 1:resIBD1[[p2]][[6]]
        ab2 <- 1:resIBD1[[p2]][[7]]

        a1 <- ab1[-na1]
        a2 <- ab2[-na2]

        if (length(a1)>0) {
          vv[[8]] <- c(vv[[8]],resIBD1[[p2]][[8]][a1])
          vv[[10]] <- c(vv[[10]],resIBD1[[p2]][[10]][a1])
          vv[[11]] <- c(vv[[11]],resIBD1[[p2]][[11]][a1])
          vv[[12]] <- c(vv[[12]],resIBD1[[p2]][[12]][a1])
          vv[[13]] <- c(vv[[13]],resIBD1[[p2]][[13]][a1])
          vv[[21]] <- c(vv[[21]],resIBD1[[p2]][[21]][a1])
        }

        if (length(a2)>0) {
          vv[[9]] <- c(vv[[9]],resIBD1[[p2]][[9]][a2])
          vv[[15]] <- c(vv[[15]],resIBD1[[p2]][[15]][a2])
          vv[[16]] <- c(vv[[16]],resIBD1[[p2]][[16]][a2])
          vv[[17]] <- c(vv[[17]],resIBD1[[p2]][[17]][a2])
          vv[[18]] <- c(vv[[18]],resIBD1[[p2]][[18]][a2])
          vv[[19]] <- c(vv[[19]],resIBD1[[p2]][[19]][a2])
          vv[[20]] <- c(vv[[20]],resIBD1[[p2]][[20]][a2])
          vv[[22]] <- c(vv[[22]],resIBD1[[p2]][[22]][a2])
        }

        vv[[14]] <- c(vv[[14]],resIBD1[[p2]][[14]])


      } else {
        if ((l2>0)&&(p2<=(l+l2))) {
          p2 <- p2-l

          ri1 <- resIBD2[[p2]][[8]]
          ri2 <- resIBD2[[p2]][[9]]

          ma1 <- match(ri1,vv[[8]])
          ma2 <- match(ri2,vv[[9]])
          na1 <- which(ma1>0)
          na2 <- which(ma2>0)

          if (length(na1)>0) {
            ma1 <- ma1[na1]
            vv[[21]][ma1] <- pmax(vv[[21]][ma1],resIBD2[[p2]][[21]][na1])
          }

          if (length(na2)>0) {
            ma2 <- ma2[na2]
            vv[[22]][ma2] <- pmax(vv[[22]][ma2],resIBD2[[p2]][[22]][na2])
          }


          ab1 <- 1:resIBD2[[p2]][[6]]
          ab2 <- 1:resIBD2[[p2]][[7]]

          a1 <- ab1[-na1]
          a2 <- ab2[-na2]

          if (length(a1)>0) {
            vv[[8]] <- c(vv[[8]],resIBD2[[p2]][[8]][a1])
            vv[[10]] <- c(vv[[10]],resIBD2[[p2]][[10]][a1])
            vv[[11]] <- c(vv[[11]],resIBD2[[p2]][[11]][a1])
            vv[[12]] <- c(vv[[12]],resIBD2[[p2]][[12]][a1])
            vv[[13]] <- c(vv[[13]],resIBD2[[p2]][[13]][a1])
            vv[[21]] <- c(vv[[21]],resIBD2[[p2]][[21]][a1])
          }

          if (length(a2)>0) {
            vv[[9]] <- c(vv[[9]],resIBD2[[p2]][[9]][a2])
            vv[[15]] <- c(vv[[15]],resIBD2[[p2]][[15]][a2])
            vv[[16]] <- c(vv[[16]],resIBD2[[p2]][[16]][a2])
            vv[[17]] <- c(vv[[17]],resIBD2[[p2]][[17]][a2])
            vv[[18]] <- c(vv[[18]],resIBD2[[p2]][[18]][a2])
            vv[[19]] <- c(vv[[19]],resIBD2[[p2]][[19]][a2])
            vv[[20]] <- c(vv[[20]],resIBD2[[p2]][[20]][a2])
            vv[[22]] <- c(vv[[22]],resIBD2[[p2]][[22]][a2])
          }

          vv[[14]] <- c(vv[[14]],resIBD2[[p2]][[14]])

        }
      }



    }


    so8 <- sort(vv[[8]],index.return=TRUE)

    vv[[8]]  <- vv[[8]][so8$ix]
    vv[[10]] <- vv[[10]][so8$ix]
    vv[[11]] <- vv[[11]][so8$ix]
    vv[[12]] <- vv[[12]][so8$ix]
    vv[[13]] <- vv[[13]][so8$ix]
    vv[[21]] <- vv[[21]][so8$ix]

    so9 <- sort(vv[[9]],index.return=TRUE)


    vv[[9]]  <- vv[[9]][so9$ix]
    vv[[15]] <- vv[[15]][so9$ix]
    vv[[16]] <- vv[[16]][so9$ix]
    vv[[17]] <- vv[[17]][so9$ix]
    vv[[18]] <- vv[[18]][so9$ix]
    vv[[19]] <- vv[[19]][so9$ix]
    vv[[20]] <- vv[[20]][so9$ix]
    vv[[22]] <- vv[[22]][so9$ix]


    vv[[4]] <- round(median(vv[[15]]))
    vv[[5]] <- max(vv[[9]])-min(vv[[9]])
    vv[[6]] <- length(vv[[8]])
    vv[[7]] <- length(vv[[9]])
    vv[[14]] <- unique(vv[[14]])

  }




   vv[[1]] <- i
   vv[[2]] <- ltr1

   res_t <- list(number=vv[[1]],bicluster=vv[[2]],chrom=vv[[3]],ibdPos=vv[[4]],ibdLength=vv[[5]],numberSamples=vv[[6]],numberSnps=vv[[7]],noSamples=vv[[8]],noSnps=vv[[9]],countrySamples=vv[[10]],idSamples=vv[[11]],labelSamples=vv[[12]],techSamples=vv[[13]],maxSamples=vv[[14]],snpPositions=vv[[15]],snpAlleles=vv[[16]],snpNames=vv[[17]],snpFreq=vv[[18]],snpGroupFreq=vv[[19]],snpChange=vv[[20]],snpsPerSample=vv[[21]],samplePerSnp=vv[[22]])


   resIBDmerge[[i]] <- res_t



}

 return(resIBDmerge)

}




iterateSegments <- function(startRun=1,endRun=640,shift=5000,segmentSize=10000,annotationFile=NULL,fileName="ALL.chr1.phase1_release_v2.20101123.snps_indels_svs",prefixPath="",sparseMatrixPostfix="_mat",annotPostfix="_annot.txt",samplesPostfix="_samples.txt",samples=0,lowerBP=0,upperBP=0.05,p=10,iter=40,quant=0.01,eps=1e-5,alpha=0.03,cyc=50,non_negative=1,write_file=0,norm=0,lap=100.0,IBDlength=50,Lt = 0.1,Zt = 0.2,thresCount=1e-5,minSNPsFactor=3/4,pMAF=0.035,haplotypes=TRUE) {


labelsA <- c()
annot <-  c()

ina <- as.numeric(readLines(paste(prefixPath,fileName,sparseMatrixPostfix,".txt",sep=""),n=2))
samplesN <- ina[1]
snps <- ina[2]

save(samplesN,snps,file=paste(fileName,"_All",".Rda",sep=""))


for (posAll in startRun:endRun)  {

start <- (posAll-1)*shift
end <- start + segmentSize

if (end > snps)  {

  end <- snps
}


pRange <- paste("_",format(start,scientific=FALSE),"_",format(end,scientific=FALSE),sep="")


if (is.null(annotationFile)) {
    labelsAA <- read.table(paste(prefixPath,fileName,samplesPostfix,sep=""),header = FALSE, sep = " ", quote = "",as.is = TRUE)
    if (haplotypes) {
        lA <- as.vector(unlist(rbind(labelsAA[,2],labelsAA[,2])))
    } else {
        lA <- as.vector(labelsAA[,2])
    }
    indiA <-  cbind(as.character(lA),as.character(lA),as.character(lA),as.character(lA))
} else {
    indit <- read.table(annotationFile, header = FALSE, sep = "\t", quote = "",as.is=TRUE)
    if (haplotypes) {
        indi1 <- as.vector(unlist(rbind(indit[,1],indit[,1]))) # because haplotypes individuals are doubled
        indi2 <- as.vector(unlist(rbind(indit[,2],indit[,2])))
        indi3 <- as.vector(unlist(rbind(indit[,3],indit[,3])))
        indi4 <- as.vector(unlist(rbind(indit[,4],indit[,4])))
     } else {
        indi1 <- as.vector(indit[,1])
        indi2 <- as.vector(indit[,2])
        indi3 <- as.vector(indit[,3])
        indi4 <- as.vector(indit[,4])
     }
    indi4 <- gsub(",",";",indi4)
    indiA <- cbind(indi1,indi2,indi3,indi4)
    labelsA <- indiA
}

# Here other labels might be possible:
# labelsA[,i]: 1=id 2=subPopulation 3=population 4 = technology

#indit <- read.table("phase1_integrated_calls.20101123.ALL.panel", header = FALSE, sep = "\t", quote = "",as.is=TRUE)
#indi1 <- as.vector(unlist(rbind(indit[,1],indit[,1]))) # because haplotypes individuals are doubled
#indi2 <- as.vector(unlist(rbind(indit[,2],indit[,2])))
#indi3 <- as.vector(unlist(rbind(indit[,3],indit[,3])))
#indi4 <- as.vector(unlist(rbind(indit[,4],indit[,4])))
#indi4 <- gsub(",",";",indi4)
#indiA <- cbind(indi1,indi2,indi3,indi4)

resHapFabia <- hapFabia(fileName=fileName,prefixPath=prefixPath,sparseMatrixPostfix=sparseMatrixPostfix,annotPostfix=annotPostfix,samplesPostfix=samplesPostfix,labelsA=labelsA,pRange=pRange,samples=samples,lowerBP=lowerBP,upperBP=upperBP,p=p,iter=iter,quant=quant,eps=eps,alpha=alpha,cyc=cyc,non_negative=non_negative,write_file=write_file,norm=norm,lap=lap,IBDlength=IBDlength,Lt = Lt,Zt = Zt,thresCount=thresCount,minSNPsFactor=minSNPsFactor,pMAF=pMAF,haplotypes=haplotypes)


ibd2excel(resHapFabia$mergedIBD,paste(fileName,pRange,".csv",sep=""))

save(resHapFabia,annot,file=paste(fileName,pRange,"_resAnno",".Rda",sep=""))

}


}



identifyDuplicates <- function(fileName="ALL.chr1.phase1_release_v2.20101123.snps_indels_svs",startRun=1,endRun=640,shift=5000,segmentSize=10000) {


snps <- 0
resHapFabia <- list()
labelsA <- c()
annot <-  c()

load(file=paste(fileName,"_All",".Rda",sep=""))

avibdLength <- list()
avibdPos <- list()
count <- 0

for (posAll in startRun:endRun)  {

start <- (posAll-1)*shift
end <- start + segmentSize

if (end > snps)  {

  end <- snps
}


pRange <- paste("_",format(start,scientific=FALSE),"_",format(end,scientific=FALSE),sep="")


load(file=paste(fileName,pRange,"_resAnno.Rda",sep=""))

mergedIBD <- resHapFabia$mergedIBD

noIBD <- length(mergedIBD)


if (noIBD>0)  {

count <- count + noIBD

avibdPos[[posAll]] <-  sapply(mergedIBD,function(x) {x[[4]]}  , simplify=FALSE)

avibdLength[[posAll]] <-  sapply(mergedIBD,function(x) {x[[5]]}  , simplify=FALSE)

}
}


ibdPos <- unlist(avibdPos)
ibdLength <- unlist(avibdLength)

ibdSim <- cbind(ibdPos,ibdLength)

dups <- duplicated(ibdSim)

un <- which(dups==FALSE)




#### enumerate counts ####




allCount <- 0
allCount1 <- 0

resD <- list()
resDA <- list()


for (posAll in startRun:endRun)  {

start <- (posAll-1)*shift
end <- start + segmentSize

if (end > snps)  {

  end <- snps
}



pRange <- paste("_",format(start,scientific=FALSE),"_",format(end,scientific=FALSE),sep="")

load(file=paste(fileName,pRange,"_resAnno.Rda",sep=""))

mergedIBD <- resHapFabia$mergedIBD

noIBD <- length(mergedIBD)

if (noIBD>0)  {

for (ibdC in 1:noIBD)  {

allCount <- allCount + 1

resD1 <- c(allCount,ibdC,posAll)
resDA[[allCount]] <- resD1

if (!dups[allCount]) {

allCount1 <- allCount1 + 1

resD1 <- c(allCount1,allCount,ibdC,posAll)
resD[[allCount1]] <- resD1



}


}

}


}


if ( length(resD) >0 ) {
rr <- unlist(resD)
l <-4

countsA1 <- matrix(rr,nrow=allCount1,ncol=l,byrow=TRUE)

colnames(countsA1) <- c("allCount1","allCount","ibdC","posAll")

rr <- unlist(resDA)
l <-3

countsA2 <- matrix(rr,nrow=allCount,ncol=l,byrow=TRUE)

colnames(countsA2) <- c("allCount","ibdC","posAll")

} else {

dups <- FALSE
un <- 0
countsA1 <- c(0,0,0,0)
dim(countsA1) <- c(1,4)
colnames(countsA1) <- c("allCount1","allCount","ibdC","posAll")
countsA2 <- c(0,0,0)
dim(countsA2) <- c(1,3)
colnames(countsA2) <- c("allCount","ibdC","posAll")

}

# write.table(countsA1,file=paste("countsA1",runIndex,".txt",sep=""))
# write.table(countsA2,file=paste("countsA2",runIndex,".txt",sep=""))



save(dups,un,countsA1,countsA2,file=paste("dups.Rda",sep=""))


}


analyzeIBDs <- function(fileName="ALL.chr1.phase1_release_v2.20101123.snps_indels_svs",runIndex="",annotPostfix="_annot.txt",startRun=1,endRun=640,shift=5000,segmentSize=10000) {

resHapFabia <- list()
countsA2 <- c()
snps <- 0
mergedIBD <- list()
dups <- c()
load(file=paste(fileName,"_All",".Rda",sep=""))
load(file=paste("dups.Rda",sep=""))

if ((startRun>1)&&(length(countsA2[,3])>1)) {
  tzz <- countsA2[which(countsA2[,3]<startRun),]
  offC <- max(tzz[,1])
} else {
  offC <-  0
}


avibdPos <- c()
avibdLength <- c()
avnoSamp <- c()
avnoSnps <- c()

avnoFreq <- c()
avnoGroupFreq <- c()
avnosnpChange <- c()
avnosnpsPerSample <- c()
avnosamplePerSnp <- c()



allCount <- 0
allCount1 <- 0

for (posAll in startRun:endRun)  {

start <- (posAll-1)*shift
end <- start + segmentSize

if (end > snps)  {

  end <- snps
}



pRange <- paste("_",format(start,scientific=FALSE),"_",format(end,scientific=FALSE),sep="")

load(file=paste(fileName,pRange,"_resAnno.Rda",sep=""))

mergedIBD <- resHapFabia$mergedIBD

noIBD <- length(mergedIBD)




if (noIBD > 0) {

for (ibdC in 1:noIBD)  {

allCount <- allCount + 1

if (!dups[allCount+offC]) {

allCount1 <- allCount1 + 1

avibdPos <- c(avibdPos,mergedIBD[[ibdC]][[4]])
avibdLength <- c(avibdLength,mergedIBD[[ibdC]][[5]])
avnoSamp <- c(avnoSamp,mergedIBD[[ibdC]][[6]])
avnoSnps <- c(avnoSnps,mergedIBD[[ibdC]][[7]])

# To annotate SNPs using matrix annotatedSNPs with SNP position at first column use this:
# snpPositions <- mergedIBD[[ibdC]][[15]]
# annotMatchT <- match(snpPositions,annotatedSNPs[,1],nomatch=0)
# snpMatch <- which(annotMatchT>0)
# annotMatch <- annotMatchT[which(annotMatchT>0)]
# matchPositionIBD <- snpPositions[snpMatch]
# annotatedSNPsIBD <- annotatedSNPs[annotMatch,]

avnoFreq <- c(avnoFreq,mergedIBD[[ibdC]][[18]])
avnoGroupFreq <- c(avnoGroupFreq,mergedIBD[[ibdC]][[19]])
avnosnpChange <- c(avnosnpChange,mergedIBD[[ibdC]][[20]])
avnosnpsPerSample <- c(avnosnpsPerSample,mergedIBD[[ibdC]][[21]])
avnosamplePerSnp <- c(avnosamplePerSnp,mergedIBD[[ibdC]][[22]])

}
}



}

}

noIBD <- allCount1


avibdPosS <- summary(avibdPos)
avibdLengthS <- summary(avibdLength)
avnoSampS <- summary(avnoSamp)
avnoSnpsS <- summary(avnoSnps)

avnoFreqS <- summary(avnoFreq)
avnoGroupFreqS <- summary(avnoGroupFreq)
avnosnpChangeS <- summary(avnosnpChange)
avnosnpsPerSampleS <- summary(avnosnpsPerSample)
avnosamplePerSnpS <- summary(avnosamplePerSnp)



save(startRun,endRun,noIBD,avibdPos,avibdLength,avnoSamp,avnoSnps,avnoFreq,avnoGroupFreq,avnosnpChange,avnosnpsPerSample,avnosamplePerSnp,avibdPosS,avibdLengthS,avnoSampS,avnoSnpsS,avnoFreqS,avnoGroupFreqS,avnosnpChangeS,avnosnpsPerSampleS,avnosamplePerSnpS,file=paste("analyzeResult",runIndex,".Rda",sep=""))

return(list(startRun=startRun,endRun=endRun,noIBD=noIBD,avibdPos=avibdPos,avibdLength=avibdLength,avnoSamp=avnoSamp,avnoSnps=avnoSnps,avnoFreq=avnoFreq,avnoGroupFreq=avnoGroupFreq,avnosnpChange=avnosnpChange,avnosnpsPerSample=avnosnpsPerSample,avnosamplePerSnp=avnosamplePerSnp,avibdPosS=avibdPosS,avibdLengthS=avibdLengthS,avnoSampS=avnoSampS,avnoSnpsS=avnoSnpsS,avnoFreqS=avnoFreqS,avnoGroupFreqS=avnoGroupFreqS,avnosnpChangeS=avnosnpChangeS,avnosnpsPerSampleS=avnosnpsPerSampleS,avnosamplePerSnpS=avnosamplePerSnpS))



}


hapFabia <- function(fileName="ALL.chr1.phase1_release_v2.20101123.snps_indels_svs",prefixPath="",sparseMatrixPostfix="_mat",annotPostfix="_annot.txt",samplesPostfix="_samples.txt",labelsA=NULL,pRange="",samples=0,lowerBP=0,upperBP=0.05,p=10,iter=40,quant=0.01,eps=1e-5,alpha=0.03,cyc=50,non_negative=1,write_file=0,norm=0,lap=100.0,IBDlength=50,Lt = 0.1,Zt = 0.2,thresCount=1e-5,minSNPsFactor=3/4,pMAF=0.035,haplotypes=TRUE) {


# fileName:            the file name of the sparse matrix in sparse format.
# prefixPath:          path of the data file
# sparseMatrixPostfix: postfix string for the sparse matrix
# annotPostfix:        postfix string for the annotation file
# labelsA:             sample names as matrix samples x 4
# prange:              for segments indicates the segment
# samples:             vector of samples which should be included into the analysis; default = 0 (all samples)
# lowerBP:             lower bound for filtering the inputs columns, minimal MAF (however more than one occurence to remove private SNPs)
# upperBP:             Upper bound for filtering the inputs columns, minimal MAF
# p:                   no biclusters per iteration
# alpha:               sparseness loadings; default = 0.03
# iter:                number iterations
# quant:               percentage of Ls to remove in each iteration
# eps:                 lower bound for variational parameter lapla; default: 1e-5
# cyc:                 number of iterations; default = 50
# non_negative:        Non-negative factors and loadings if non_negative; default = 1 (yes).
# write_file:          results are written to files (L in sparse format), default = 0 (not written).
# norm:                data normalization; default = 1 (no normalization).
# lap:                 minimal value of the variational parameter; default = 100.0.
# IBDlength:           IBD or haplotype length in kbp
# Lt:                  percentage of largest Ls to consider for IBD extraction
# Zt:                  percentage of largest Zs to consider for IBD extraction
# thresCount:          p-value of random histogram hit, default 1e-5
# minSNPsFactor:       percentage of IBD overlap; 1/2 for large to 3/4 for for small intervals
# pMAF:                averaged and corrected minor allele frequency
# haplotypes:          haplotypes = phased genotypes -> two chromosomes per individual


message("Running hapFabia with:")
message("   File name of the sparse matrix in sparse format --- : ",fileName)
message("   Path of the data file ------------------------------: ",prefixPath)
message("   Postfix string for sparse matrix format ------------: ",sparseMatrixPostfix)
message("   Postfix string for file containing sample names ----: ",samplesPostfix)
if (is.null(labelsA)) {
message("   Samples annotation not given -----------------------")
} else {
message("   Samples annotation given ////-----------------------")
}
message("   String indicating the segment that is analyzed -----: ",pRange)
if (length(samples)>1) {
message("   Number of samples included into the analysis -------: ",length(samples))
} else {
message("   All samples are included into the analysis ---------: 0 = all samples")
}
message("   Lower bound on MAF but more than one occurence -----: ",lowerBP  )
message("   Upper bound on MAF ---------------------------------: ",upperBP)
message("   Number of biclusters per iteration -----------------: ",p)
message("   Sparseness coefficient of the loadings -------------: ",alpha)
message("   Number of iterations -------------------------------: ",iter)
message("   Percentage of Ls to remove after each iteration ----: ",quant)
message("   Lower bound for variational parameter lapla --------: ",eps)
message("   Number of cycles per iteration ---------------------: ",cyc)
message("   Non-negative factors and loadings ------------------: ",non_negative)
message("   Results written to files ---------------------------: ",write_file)
message("   Data normalized ------------------------------------: ",norm)
message("   Minimal value of the variational parameter ---------: ",lap)
message("   IBD or haplotype length in kbp ---------------------: ",IBDlength)
message("   Percentage of largest Ls for haplotye extraction ---: ",Lt)
message("   Percentage of largest Zs for haplotye extraction ---: ",Zt)
message("   p-value threshold of random histogram hit ----------: ",thresCount)
message("   Percentage of haplotype overlap --------------------: ",minSNPsFactor)
message("   Averaged and corrected minor allele frequency ------: ",pMAF)
if (haplotypes) {
message("   Data consists of phased genotypes / haplotypes -----")
} else {
message("   Data consists of unphased genotypes ----------------")
}




require("fabia")


# Compute internal parameters

# ps: quantile above which to consider Ls
ps <- 1 - Lt
# psZ: quantile above which to consider Zs
psZ <- 1 - Zt
inteA <- IBDlength*10 # histogram length gives inteA/10 kbp DNA length

ina <- as.numeric(readLines(paste(prefixPath,fileName,pRange,sparseMatrixPostfix,".txt",sep=""),n=2))
if (length(samples)>1) {
    samplesN <- length(samples)
} else {
    samplesN <- ina[1]
    samples <- 0
}
snps <- ina[2]

upperBsamp=upperBP*samplesN # remove common SNPs
lowerBsamp=max(1.5,lowerBP*samplesN) # remove private SNPs


kk <- 1
while ((snps/inteA)*choose(samplesN,2)*(1-pbinom(kk,inteA,pMAF*pMAF))>thresCount) { kk <- kk +1}

thresA <- kk
minSNPs <- round(minSNPsFactor*thresA)

# End Compute internal parameters



# Fabia call
# sparseMatrixPostfix="_mat" is fixed --- should be changed
res <- spfabia(X=paste(prefixPath,fileName,pRange,sparseMatrixPostfix,sep=""),p=p,alpha=alpha,cyc=cyc,non_negative=non_negative,write_file=write_file,norm=norm,lap=lap,samples=samples,iter=iter,quant=quant,lowerB=lowerBsamp,upperB=upperBsamp,eps=eps)


# Load samples to Ls of interest: load minor alleles of the Ls

# sparseMatrixPostfix="_mat" is fixed --- should be changed
sPF <- samplesPerFeature(X=paste(prefixPath,fileName,pRange,sparseMatrixPostfix,sep=""),samples=samples,lowerB=lowerBsamp,upperB=upperBsamp)

if (nchar(annotPostfix)>0) {

    annot <- read.table(paste(prefixPath,fileName,pRange,annotPostfix,sep=""),header = FALSE, sep = " ", quote = "",as.is = TRUE,skip=2)


    for (i in 1:length(annot)) {

        annot[[i]] <- gsub(",",";",annot[[i]])

    }
    for (i in 1:length(annot)) {

        annot[[i]] <- gsub("TRUE","T",annot[[i]])

    }

    annot[[2]] <- as.numeric(annot[[2]]) # physical position
    annot[[10]] <- as.numeric(annot[[10]]) # SNV frequency

}

if (is.null(labelsA)) {
    labelsAA <- read.table(paste(prefixPath,fileName,samplesPostfix,sep=""),header = FALSE, sep = " ", quote = "",as.is = TRUE)
    if (haplotypes) {
        lA <- as.vector(unlist(rbind(labelsAA[,2],labelsAA[,2])))
    } else {
        lA <- as.vector(labelsAA[,2])
    }
    indiA <-  cbind(as.character(lA),as.character(lA),as.character(lA),as.character(lA))
} else {
    indiA <- labelsA
}

# Here other labels might be possible:
# labelsA[,i]: 1=id 2=subPopulation 3=population 4 = technology

#indit <- read.table("phase1_integrated_calls.20101123.ALL.panel", header = FALSE, sep = "\t", quote = "",as.is=TRUE)
#indi1 <- as.vector(unlist(rbind(indit[,1],indit[,1]))) # because haplotypes individuals are doubled
#indi2 <- as.vector(unlist(rbind(indit[,2],indit[,2])))
#indi3 <- as.vector(unlist(rbind(indit[,3],indit[,3])))
#indi4 <- as.vector(unlist(rbind(indit[,4],indit[,4])))
#indi4 <- gsub(",",";",indi4)
#indiA <- cbind(indi1,indi2,indi3,indi4)



# save fabia result
# save(res,sPF,annot,file=paste(fileName,pRange,"_res.Rda",sep=""))


# first haplotype extraction with offset 0

off1 <- 0

resIBD1 <- getIBDs(res=res,sPF=sPF,annot=annot,chrom=1,labelsA=indiA,ps=ps,inteA=inteA,thresA=thresA,minSNPs=minSNPs,off=off1)

# merge haplotypes

if ( length(resIBD1) > 1) {
  comp <- compareIBDres(resIBD1,simv="minD")

  if (!is.null(comp)) {
      mergedIBD1 <- mergeIBDres(resIBD1=resIBD1,resIBD2=NULL,comp=comp,cut=0.8)
  } else {
      mergedIBD1 <- resIBD1
  }
} else {

 mergedIBD1 <- resIBD1
}


# second haplotype extraction with offset half of the interval length


off2=inteA%/%2

resIBD2 <- getIBDs(res=res,sPF=sPF,annot=annot,chrom=1,labelsA=indiA,ps=ps,inteA=inteA,thresA=thresA,minSNPs=minSNPs,off=off2)

# merge haplotypes

if ( length(resIBD2) > 1) {
  comp <- compareIBDres(resIBD2,simv="minD")

  if (!is.null(comp)) {
      mergedIBD2 <- mergeIBDres(resIBD1=resIBD2,resIBD2=NULL,comp=comp,cut=0.8)
  } else {
      mergedIBD2 <- resIBD2
  }
} else {
  mergedIBD2 <- resIBD2
}

# merge haplotypes of both extractions


if ( length(mergedIBD1) > 0) {

  if ( length(mergedIBD2) > 0) {

    comp12 <- compareIBDres(mergedIBD1,mergedIBD2,simv="minD")

    if (!is.null(comp12)) {
        mergedIBD <- mergeIBDres(resIBD1=mergedIBD1,resIBD2=mergedIBD2,comp=comp12,cut=0.8)
    } else {
        mergedIBD <- mergedIBD1
    }

} else {

mergedIBD <- mergedIBD1
}

} else {

  if ( length(mergedIBD2) > 0) {
    mergedIBD <- mergedIBD2
  } else {
    mergedIBD <- mergedIBD1
  }


}



save(mergedIBD,res,sPF,annot,resIBD1,resIBD2,mergedIBD1,mergedIBD2,file=paste(fileName,pRange,"_resIBD.Rda",sep=""))

return(list(mergedIBD=mergedIBD,res=res,sPF=sPF,annot=annot,resIBD1=resIBD1,resIBD2=resIBD2,mergedIBD1=mergedIBD1,mergedIBD2=mergedIBD2))

}





simulateHaplotypes <- function(fileprefix="dataSim",minruns=1,maxruns=100,snps=10000,samplesN=100,avDistSnps=100,avDistMinor=25,noImplanted=1,implanted=10,length=100,minors=20,mismatches=0,mismatchImplanted=0.5,overlap=50,noOverwrite=FALSE) {

## minruns: min number or runs
## maxruns: max number or runs
## snps: number of SNPs
## samplesN: number of individuals
## avDistSnps: average distance in bases between SNPs
## avDistMinor: average distance between minor alleles, thus 1/avDistMinor is the average MAF
## noImplanted: how many haplotypes
## implanted: in how many samples is one haplotype implanted
## length: length of haplotypes
## minors: number of tagSNVs
## mismatches: number of mismatches in hapltopye
## mismatchImplanted: mismatches in how many percent of haplotypes
## overlap: minimal haplotype overlap between chromosomes
## noOverwrite: noOverwrite=TRUE ensures that a haplotype is not superimposed by another haplotype

write(paste("maxruns: ",maxruns,sep=""),file=paste(fileprefix,"Parameters.txt",sep=""),ncolumns=100)
write(paste("snps: ",snps,sep=""),file=paste(fileprefix,"Parameters.txt",sep=""),append=TRUE,ncolumns=100)
write(paste("samplesN: ",samplesN,sep=""),file=paste(fileprefix,"Parameters.txt",sep=""),append=TRUE,ncolumns=100)
write(paste("avDistSnps: ",avDistSnps,sep=""),file=paste(fileprefix,"Parameters.txt",sep=""),append=TRUE,ncolumns=100)
write(paste("avDistMinor: ",avDistMinor,sep=""),file=paste(fileprefix,"Parameters.txt",sep=""),append=TRUE,ncolumns=100)
write(paste("noImplanted: ",noImplanted,sep=""),file=paste(fileprefix,"Parameters.txt",sep=""),append=TRUE,ncolumns=100)
write(paste("implanted: ",implanted,sep=""),file=paste(fileprefix,"Parameters.txt",sep=""),append=TRUE,ncolumns=100)
write(paste("length: ",length,sep=""),file=paste(fileprefix,"Parameters.txt",sep=""),append=TRUE,ncolumns=100)
write(paste("minors: ",minors,sep=""),file=paste(fileprefix,"Parameters.txt",sep=""),append=TRUE,ncolumns=100)
write(paste("mismatches: ",mismatches,sep=""),file=paste(fileprefix,"Parameters.txt",sep=""),append=TRUE,ncolumns=100)
if (mismatches>0) {
  write(paste("mismatchImplanted: ",mismatchImplanted,sep=""),file=paste(fileprefix,"Parameters.txt",sep=""),append=TRUE,ncolumns=100)
}
write(paste("overlap: ",overlap,sep=""),file=paste(fileprefix,"Parameters.txt",sep=""),append=TRUE,ncolumns=100)


rateSnps <- 1/avDistSnps

rateMinor <- 1/avDistMinor



haploN <- 2*samplesN

dataA <- 1:samplesN


nucleotide <- c("A","T","C","G")




physPos <- vector("integer",snps)
snpMajor <- vector("character",snps)
snpMinor <- vector("character",snps)

g2 <- 2*(1:snps)
g1 <- g2 -1


s2 <- 2*dataA
s1 <- s2-1


id <- as.character(dataA)
zeroid <-  as.character(rep(0,samplesN))

famID <- id
dim(famID) <- c(samplesN,1)
ID <- id
dim(ID) <- c(samplesN,1)
patID <- zeroid
dim(patID) <- c(samplesN,1)
matID <- zeroid
dim(matID) <- c(samplesN,1)
sex <- zeroid
dim(sex) <- c(samplesN,1)
phen <- zeroid
dim(phen) <- c(samplesN,1)



snpNames <- as.character(1:snps)

chr <- rep(1,snps)

alleleI <- matrix(0,haploN,snps)
alleleN <- matrix("N",haploN,snps)


#################

for (run in minruns:maxruns) {

  write(noImplanted,file=paste(fileprefix,run,"Impl.txt",sep=""),ncolumns=100)


  distsSnps <- rexp(snps,rateSnps)

  aa <- as.integer(0)
  for (i in 1:snps) {
    aa <- aa + 1 + as.integer(round(distsSnps[i]))
    physPos[i] <- aa

    mami <- sample(4,2)
    snpMajor[i] <- nucleotide[mami[1]]
    snpMinor[i] <- nucleotide[mami[2]]
  }


  for (s in 1:haploN) {
    for (i in 1:snps) {
      if (runif(1)<rateMinor) {
        alleleI[s,i] <- 1
        alleleN[s,i] <- snpMinor[i]
      } else {
        alleleI[s,i] <- 0
        alleleN[s,i] <- snpMajor[i]
      }
    }
  }



  freq <- colSums(alleleI)/haploN
  pos <- physPos
  pos1 <- pos/1000000




  plinkCols <- cbind(famID,ID,patID,matID,sex,phen)

  plinkmap <- cbind(chr,snpNames,pos1,pos)

  plinkmap[,1] <- as.integer(plinkmap[,1])
  plinkmap[,2] <- as.integer(plinkmap[,2])
  plinkmap[,3] <- format(as.double(plinkmap[,3]),nsmall=6)
  plinkmap[,4] <- as.integer(plinkmap[,4])

  plinkLine <- c("FID","IID","PAT","MAT","SEX","PHENOTYPE",snpNames)


  mcmc <- rbind(pos1,freq)

  initM <- matrix(0,samplesN,2*snps)


  alleleIimp <- alleleI
  alleleNimp <- alleleN

  allinter <- list()
  allimpl <- list()

  for (noIm in 1:noImplanted) {

  notfoundIBD <- TRUE
  while (notfoundIBD) {


  start <- sample((snps-length-1),1)

  end <- start+length-1

  inter <- start:end

  allinter[[noIm]] <- inter

  posMinor <- sample(length,minors)

  implantI <- rep(0,length)

  implantI[posMinor] <- 1

  implantN <- rep("N",length)

  ll <- 1
  for (i in inter) {
    if (implantI[ll]==0) {
      implantN[ll] <- snpMajor[i]
    } else {
      implantN[ll] <- snpMinor[i]
    }
    ll <- ll + 1
  }


  impl <- sample(haploN,implanted)

  allimpl[[noIm]] <- impl

  notfoundIBD <- FALSE
  if ((noIm>1)&&(noOverwrite)) {
    for (no2Im in 1:(noIm-1)) {
      if ((length(intersect(impl,allimpl[[no2Im]]))>0)&&(length(intersect(inter,allinter[[no2Im]]))>0)) {
        notfoundIBD <- TRUE
      }
    }
  }


  }


  impl=sort(impl)

  implG <- (impl+1)%/%2
  write(impl,file=paste(fileprefix,run,"Impl.txt",sep=""),append=TRUE,ncolumns=100)
  write(implG,file=paste(fileprefix,run,"Impl.txt",sep=""),append=TRUE,ncolumns=100)
  write(inter,file=paste(fileprefix,run,"Impl.txt",sep=""),append=TRUE,ncolumns=100)
  write(implantI,file=paste(fileprefix,run,"Impl.txt",sep=""),append=TRUE,ncolumns=100)
  write(implantN,file=paste(fileprefix,run,"Impl.txt",sep=""),append=TRUE,ncolumns=100)

  intervalImp <- matrix(0,implanted,4)


  for (i in 1:implanted) {


    startI <- 1+sample((length-overlap)%/%2,1)
    endI <- length - 1 - sample((length-overlap)%/%2,1)
    interI <- inter[startI:endI]
    alleleIimp[impl[i],interI] <- implantI[startI:endI]
    alleleNimp[impl[i],interI] <- implantN[startI:endI]

    interImp <- c(inter[startI],inter[endI],startI,endI)
    intervalImp[i,] <-  interImp

    write(interImp,file=paste(fileprefix,run,"Impl.txt",sep=""),append=TRUE,ncolumns=100)
  }

  if (mismatches > 1) {

    posMismatcht <- 1+ sample((length-2),mismatches)

    posMismatch <- inter[posMismatcht]

    noMisSamples <- round(mismatchImplanted*implanted)


    for (i1 in posMismatch) {
      sampind <- sample(implanted,noMisSamples)
      for (i1S in sampind) {
        if ((i1>=intervalImp[i1S,3])&&(i1<=intervalImp[i1S,4])) {
          whichSample <- impl[i1S]
          if (alleleIimp[whichSample,i1]==0) {
            alleleIimp[whichSample,i1] <- 1
            alleleNimp[whichSample,i1] <- snpMinor[i1]
          } else {
            alleleIimp[whichSample,i1] <- 0
            alleleNimp[whichSample,i1] <- snpMajor[i1]
          }
        }

      }

    }
  }


}


alleleIimpGeno <- alleleIimp[s1,] + alleleIimp[s2,]

dummyL <- 1:snps
dummyC <- rep(0,snps)
dummy <- as.character(dummyL)

annot <- list()

annot[[1]] <- dummyL
annot[[2]] <- pos
annot[[3]] <- snpNames
annot[[4]] <- snpMajor
annot[[5]] <- snpMinor
annot[[6]] <- dummy
annot[[7]] <- dummy
annot[[8]] <- dummy
annot[[9]] <- dummy
annot[[10]] <- freq
annot[[11]] <- dummyC


 save(snps,haploN,samplesN,dataA,pos,pos1,snpNames,snpMajor,snpMinor,freq,annot,file=paste(fileprefix,run,"Anno.Rda",sep=""))

 save(impl,implG,inter,implantI,implantN,intervalImp,file=paste(fileprefix,run,"Impl.Rda",sep=""))

  # BEAGLE
  ########

  v1 <- rep(1,haploN)
  dim(v1) <- c(1,haploN)

  v0 <- as.numeric(rbind(dataA,dataA))
  dim(v0) <- c(1,haploN)



  dataB1 <- rbind(v0,v1,t(alleleNimp))

  col1 <- c("id","BC58",snpNames)

  dim(col1) <- c((snps+2),1)

  col2 <- rep("M",snps)

  col2 <- c("I","A",col2)

  dim(col2) <- c((snps+2),1)


  dataB <- cbind(col2,col1,dataB1)

  write.table(dataB,file=paste(fileprefix,run,"beagle.txt",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)




  # PLINK
  #######

  dataP1 <- matrix("character",nrow=samplesN,ncol=2*snps)


  dataP1[,g1] <- alleleNimp[s1,]
  dataP1[,g2] <- alleleNimp[s2,]


  dataP <- cbind(plinkCols,dataP1)

  write.table(dataP,file=paste(fileprefix,run,"plink.ped",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)



  write.table(plinkmap,file=paste(fileprefix,run,"plink.map",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE,sep="\t")


  write.table(plinkCols,file=paste(fileprefix,run,"plink.fam",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE,sep="\t")






   #MCMC
   #####

  write.table(alleleIimpGeno,file=paste(fileprefix,run,"mcmc.genotype",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE,sep="\t")
  write.table(mcmc,file=paste(fileprefix,run,"mcmc.posmaf",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE,sep="\t")
  write.table(initM,file=paste(fileprefix,run,"mcmc.initz",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE,sep="\t")


   #RELATE
   ########


  write.table((alleleIimpGeno+1),file=paste(fileprefix,run,"relate.geno",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)

  write.table(pos1,file=paste(fileprefix,run,"relate.pos",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)
  write.table(chr,file=paste(fileprefix,run,"relate.chr",sep=""),quote=FALSE,row.names=FALSE,col.names=FALSE)


  #FABIA
  ######


  namesL <-  cbind(1:haploN,1:haploN)
  write.table(namesL,file=paste(fileprefix,run,"fabia_samples.txt",sep=""),quote = FALSE,row.names = FALSE,col.names = FALSE)


  write(as.integer(haploN),file=paste(fileprefix,run,"fabia_annot.txt",sep=""),ncolumns=100)
  write(as.integer(snps),file=paste(fileprefix,run,"fabia_annot.txt",sep=""),append=TRUE,ncolumns=100)
  write.table(annot,paste(fileprefix,run,"fabia_annot.txt",sep=""), sep = " ", quote = FALSE,row.names = FALSE,col.names = FALSE,append=TRUE)


  write(as.integer(haploN),file=paste(fileprefix,run,"fabia_mat.txt",sep=""),ncolumns=100)
  write(as.integer(snps),file=paste(fileprefix,run,"fabia_mat.txt",sep=""),append=TRUE,ncolumns=100)

  for (i in 1:haploN) {

    a1 <- which(alleleIimp[i,]>0.01)

    al <- length(a1)
    b1 <- alleleIimp[i,a1]

    a1 <- a1 - 1
    dim(a1) <- c(1,al)
    b1 <- format(as.double(b1),nsmall=1)
    dim(b1) <- c(1,al)

    write.table(al,file=paste(fileprefix,run,"fabia_mat.txt",sep=""), sep = " ", quote = FALSE,row.names = FALSE,col.names = FALSE,append=TRUE)
    write.table(a1,file=paste(fileprefix,run,"fabia_mat.txt",sep=""), sep = " ", quote = FALSE,row.names = FALSE,col.names = FALSE,append=TRUE)
    write.table(b1,file=paste(fileprefix,run,"fabia_mat.txt",sep=""), sep = " ", quote = FALSE,row.names = FALSE,col.names = FALSE,append=TRUE)

  }


}

}

split_sparse_matrix <- function(fileName="ALL.chr1.phase1_release_v2.20101123.snps_indels_svs",sparseMatrixPostfix="_mat.txt",segmentSize=10000,shiftSize=5000,annotation=TRUE) {

    narg <- as.integer(6)
    arg1 <- as.character(fileName)
    arg2 <- as.character(sparseMatrixPostfix)
    arg3 <- as.character(segmentSize)
    arg4 <- as.character(shiftSize)
    if (annotation) {
        arg5 <- as.character(1)
    } else {
        arg5 <- as.character(0)
    }

    message("Running 'split_sparse_matrix' on ",fileName)
    message("   Postfix of sparse matrix format ---- : ",sparseMatrixPostfix)
    message("   Segment size ----------------------- : ",segmentSize)
    message("   Shift size ------------------------- : ",shiftSize)
    if (annotation) {
    message("   Annotation is available ------------ : ")
    } else {
    message("   Annotation is not available -------- :")
    }

    .Call("split_sparse_matrix",narg,arg1,arg2,arg3,arg4,arg5,PACKAGE="hapFabia")

    message("")
    message("Split End.")


}


vcftoFABIA <- function(fileName="ALL.chr1.phase1_release_v2.20101123.snps_indels_svs",prefixPath="",noSnps=NULL) {


    arg1 <- as.character(fileName)
    arg2 <- as.character(prefixPath)
    if (is.null(noSnps)) {
        narg <- as.integer(3)
        arg3 <-  as.character(0)
   } else {
        narg <- as.integer(4)
        arg3 <-  as.character(noSnps)
    }

    message("Running 'vcftoFABIA' on ",fileName)
    message("   Path to file ----------------------- : ",prefixPath)
    if (narg==4) {
    message("   Number of SNVs --------------------- :",noSnps)
    } else {
    message("   Number of SNVs are unknown --------- :")
    }


    .Call("vcftoFABIA",narg,arg1,arg2,arg3,PACKAGE="hapFabia")

    message("")
    message("Convert End.")


}

simulateForFabia <- function(fileprefix="dataSim",minruns=1,maxruns=1,snps=1000,samplesN=100,avDistSnps=100,avDistMinor=10,noImplanted=1,implanted=10,length=50,minors=30,mismatches=0,mismatchImplanted=0.5,overlap=50) {


rateSnps <- 1/avDistSnps

rateMinor <- 1/avDistMinor



haploN <- 2*samplesN

dataA <- 1:samplesN



physPos <- vector("integer",snps)


alleleI <- matrix(0,haploN,snps)


#################

for (run in minruns:maxruns) {

#  write(noImplanted,file=paste(fileprefix,run,"Impl.txt",sep=""),ncolumns=100)


  distsSnps <- rexp(snps,rateSnps)

  aa <- as.integer(0)
  for (i in 1:snps) {
    aa <- aa + 1 + as.integer(round(distsSnps[i]))
    physPos[i] <- aa

  }


  for (s in 1:haploN) {
    for (i in 1:snps) {
      if (runif(1)<rateMinor) {
        alleleI[s,i] <- 1
      } else {
        alleleI[s,i] <- 0
      }
    }
  }



  freq <- colSums(alleleI)/haploN
  pos <- physPos
  pos1 <- pos/1000000


  alleleIimp <- alleleI

  allinter <- list()
  allimpl <- list()

  for (noIm in 1:noImplanted) {



  start <- sample((snps-length-1),1)

  end <- start+length-1

  inter <- start:end

   posMinor <- sample(length,minors)

  implantI <- rep(0,length)

  implantI[posMinor] <- 1


  impl <- sample(haploN,implanted)

  allimpl[[noIm]] <- impl


  impl=sort(impl)

  implG <- (impl+1)%/%2
#  write(impl,file=paste(fileprefix,run,"Impl.txt",sep=""),append=TRUE,ncolumns=100)
#  write(implG,file=paste(fileprefix,run,"Impl.txt",sep=""),append=TRUE,ncolumns=100)
#  write(inter,file=paste(fileprefix,run,"Impl.txt",sep=""),append=TRUE,ncolumns=100)
#  write(implantI,file=paste(fileprefix,run,"Impl.txt",sep=""),append=TRUE,ncolumns=100)


  intervalImp <- matrix(0,implanted,4)


  for (i in 1:implanted) {


    startI <- 1+sample((length-overlap)%/%2,1)
    endI <- length - 1 - sample((length-overlap)%/%2,1)
    interI <- inter[startI:endI]
    alleleIimp[impl[i],interI] <- implantI[startI:endI]

    interImp <- c(inter[startI],inter[endI],startI,endI)
    intervalImp[i,] <-  interImp

#    write(interImp,file=paste(fileprefix,run,"Impl.txt",sep=""),append=TRUE,ncolumns=100)
  }

  if (mismatches > 1) {

    posMismatcht <- 1+ sample((length-2),mismatches)

    posMismatch <- inter[posMismatcht]

    noMisSamples <- round(mismatchImplanted*implanted)


    for (i1 in posMismatch) {
      sampind <- sample(implanted,noMisSamples)
      for (i1S in sampind) {
        if ((i1>=intervalImp[i1S,3])&&(i1<=intervalImp[i1S,4])) {
          whichSample <- impl[i1S]
          if (alleleIimp[whichSample,i1]==0) {
            alleleIimp[whichSample,i1] <- 1
             } else {
            alleleIimp[whichSample,i1] <- 0
           }
        }

      }

    }
  }


}


dummyL <- 1:snps
dummyC <- rep(0,snps)
dummy <- as.character(dummyL)
dummyM <- rep("A",snps)
dummyU <- rep("T",snps)
snpNames <- dummy

annot <- list()

annot[[1]] <- dummyL
annot[[2]] <- pos
annot[[3]] <- snpNames
annot[[4]] <- dummyM
annot[[5]] <- dummyU
annot[[6]] <- dummy
annot[[7]] <- dummy
annot[[8]] <- dummy
annot[[9]] <- dummy
annot[[10]] <- freq
annot[[11]] <- dummyC

namesL <-  cbind(1:haploN,1:haploN)

simu <- list(namesL,haploN,snps,annot,alleleIimp)
save(simu,file=paste(fileprefix,run,"fabia_simulation.RData",sep=""))


write.table(namesL,file=paste(fileprefix,run,"fabia_samples.txt",sep=""),quote = FALSE,row.names = FALSE,col.names = FALSE)

write(as.integer(haploN),file=paste(fileprefix,run,"fabia_annot.txt",sep=""),ncolumns=100)
write(as.integer(snps),file=paste(fileprefix,run,"fabia_annot.txt",sep=""),append=TRUE,ncolumns=100)
write.table(annot,paste(fileprefix,run,"fabia_annot.txt",sep=""), sep = " ", quote = FALSE,row.names = FALSE,col.names = FALSE,append=TRUE)

# save(snps,haploN,samplesN,dataA,pos,pos1,snpNames,snpMajor,snpMinor,freq,annot,file=paste(fileprefix,run,"Anno.Rda",sep=""))

# save(impl,implG,inter,implantI,implantN,intervalImp,file=paste(fileprefix,run,"Impl.Rda",sep=""))


  write(as.integer(haploN),file=paste(fileprefix,run,"fabia_mat.txt",sep=""),ncolumns=100)
  write(as.integer(snps),file=paste(fileprefix,run,"fabia_mat.txt",sep=""),append=TRUE,ncolumns=100)

  for (i in 1:haploN) {

    a1 <- which(alleleIimp[i,]>0.01)

    al <- length(a1)
    b1 <- alleleIimp[i,a1]

    a1 <- a1 - 1
    dim(a1) <- c(1,al)
    b1 <- format(as.double(b1),nsmall=1)
    dim(b1) <- c(1,al)

    write.table(al,file=paste(fileprefix,run,"fabia_mat.txt",sep=""), sep = " ", quote = FALSE,row.names = FALSE,col.names = FALSE,append=TRUE)
    write.table(a1,file=paste(fileprefix,run,"fabia_mat.txt",sep=""), sep = " ", quote = FALSE,row.names = FALSE,col.names = FALSE,append=TRUE)
    write.table(b1,file=paste(fileprefix,run,"fabia_mat.txt",sep=""), sep = " ", quote = FALSE,row.names = FALSE,col.names = FALSE,append=TRUE)

  }


}

}

