\name{ibd2excel}
\alias{ibd2excel}
\title{ibd2excel: Store haplotype cluster or IBD tract in EXCEL format}
\description{

  \code{ibd2excel}: \R implementation of \code{ibd2excel}.


  Haplotype Cluster or IBD tracts are stored in a file in EXCEL format, more precise
  in comma separate format (.csv).

}
\usage{

ibd2excel(resIBD,filename)


}
\arguments{
  \item{resIBD}{list of haplotype clusters or IBD tracts.}
  \item{filename}{file name where to store the EXCEL format data.}

}
\details{


  Haplotype Cluster or IBD tracts are stored
  in comma separate format (.csv) which can readily be read by EXCEL.

  The EXCEL file contains following columns:

\enumerate{
  \item number: number of haplotype clusters in current extraction procedure.
  \item bicluster: ID of the bicluster the haplotype cluster was found in.
  \item chrom: the chromosome.
  \item ibdPos: genomic location of the haplotype cluster.
  \item ibdLength: length of the haplotype cluster.
  \item numberSamples: number of samples belonging to the haplotype cluster.
  \item numberSnps: number tagSNVs of the haplotype cluster.
  \item noSamples: samples IDs of samples belonging to the haplotype cluster.
  \item noSnps: SNV IDs of the haplotype cluster tagSNVs.
  \item countrySamples: second label of samples.
  \item idSamples: sample names; first label of samples.
  \item labelSamples: third label of samples.
  \item techSamples: technology used; fourth label of samples.
  \item maxSamples: IDs of samples building the core haplotype cluster.
  \item snpPositions: physical positions of the SNVs on the chromosome in bases.
  \item snpAlleles: alleles in the form \code{Ref:Alt} where Ref denotes reference allele and Alt the alternative allele.
  \item snpNames: name of SNVs according to annotation.
  \item snpFreq: frequency of SNVs in whole population.
  \item snpGroupFreq: frequency of SNVs in population that is considered.
  \item snpChange: if the minor allele was more frequent than the major, then both were switched.
  \item snpsPerSample: for each sample: tagSNVs are counted for which the sample has the minor allele.
  \item samplePerSnp: for each CNV: samples are counted for which the
  CNV has its minor allele.
}

  
  Implementation in \R.


}
\value{

data(hapRes)
mergedIBD <- hapRes$mergedIBD
ibd2excel(resIBD=mergedIBD,filename="testResult.csv")

unlink("testResult.csv")


    }
\seealso{
\code{\link{hapFabia}},
\code{\link{vcftoFABIA}},
\code{\link{split_sparse_matrix}},
\code{\link{iterateSegments}},
\code{\link{getIBDs}},
\code{\link{extractIBD}},
\code{\link{compareIBDres}},
\code{\link{mergeIBDres}},
\code{\link{identifyDuplicates}},
\code{\link{ibd2excel}},
\code{\link{plotRangeL}},
\code{\link{plotOneIBD}},
\code{\link{plotIBDs}},
\code{\link{snpPlot}},
\code{\link{analyzeIBDs}},
\code{\link{simulateHaplotypes}},
\code{\link{simulateForFabia}},
\code{\link{sim}},
\code{\link{topLZ}},
\code{\link{histL}},
\code{\link{plotL}},
\code{\link{hapRes}},
\code{\link{res}},
\code{\link{simu}},
\code{\link{mergedIBD}}
}
\author{Sepp Hochreiter}
\references{

S. Hochreiter et al.,
\sQuote{FABIA: Factor Analysis for Bicluster Acquisition},
Bioinformatics 26(12):1520-1527, 2010.

}
\keyword{file}
\concept{genetics,haplotype,identity by descent,bicluster,next generation sequencing,genotype,single nucleotide polymorphism,single nucleotide variation,rare variants,rare SNPs, rare SNVs,rare haplotype clusters,short haplotype clusters}
