\name{identifyDuplicates}
\alias{identifyDuplicates}
\title{identifyDuplicates: Identify duplicated haplotype clusters in different results}
\description{

  \code{identifyDuplicates}: \R implementation of \code{identifyDuplicates}.

  Haplotype Clusters that are identical in different results are identified. Used in combination
  with \code{iterateSegments} for splitting chromosome in overlapping segments.
  Results are written to a file.
}
\usage{

identifyDuplicates(fileName="ALL.chr1.phase1_release_v2.20101123.snps_indels_svs",startRun=1,endRun=640,shift=5000,segmentSize=10000)


}
\arguments{
 \item{fileName}{file name prefix without type of the result of \code{hapFabia}; Attention no type!}
 \item{startRun}{first segment.}
 \item{endRun}{last segment.}
 \item{shift}{distance between start of adjacent segments.}
 \item{segmentSize}{number of SNVs or SNPs in a segment.}

}
\details{

  Haplotype Clusters that are identical in different results are identified and the result written to a file.
  For analysis across the whole chromosome or genome this information is important in order to avoid
  double counting.
  Used subsequently to \code{iterateSegments} which analyzes segments of a chromosome.
  Provides important information for \code{analyzeIBDs} to avoid redundancies.

  Results are saved in "dups.Rda" file containing \code{dups} (the index of duplicates),
  \code{un} (the index of non-duplicates),
  \code{countsA1} (the counts and mapping to segments for non-duplicates), and
  \code{countsA2} (the counts and mapping to segments for all haplotype clusters).

  Implementation in \R.

}
\seealso{
\code{\link{hapFabia}},
\code{\link{vcftoFABIA}},
\code{\link{split_sparse_matrix}},
\code{\link{iterateSegments}},
\code{\link{getIBDs}},
\code{\link{extractIBD}},
\code{\link{compareIBDres}},
\code{\link{makePipelineFile}},
\code{\link{mergeIBDres}},
\code{\link{identifyDuplicates}},
\code{\link{ibd2excel}},
\code{\link{plotRangeL}},
\code{\link{plotOneIBD}},
\code{\link{plotIBDs}},
\code{\link{snpPlot}},
\code{\link{analyzeIBDs}},
\code{\link{simulateHaplotypes}},
\code{\link{simulateForFabia}},
\code{\link{sim}},
\code{\link{topLZ}},
\code{\link{histL}},
\code{\link{plotL}},
\code{\link{hapRes}},
\code{\link{res}},
\code{\link{simu}},
\code{\link{mergedIBD}}
}
\author{Sepp Hochreiter}
\references{

S. Hochreiter et al.,
\sQuote{FABIA: Factor Analysis for Bicluster Acquisition},
Bioinformatics 26(12):1520-1527, 2010.

}
\keyword{models,multivariate,cluster}
\concept{genetics,haplotype,identity by descent,bicluster,next generation sequencing,genotype,single nucleotide polymorphism,single nucleotide variation,rare variants,rare SNPs, rare SNVs,rare haplotype clusters,short haplotype clusters}
