\name{iterateSegments}
\alias{iterateSegments}
\title{iterateSegments: Loop over all segments which calls \code{hapFabia}}
\description{

  \code{iterateSegments}: \R implementation of \code{iterateSegments}.

  Loops over all segments and calls \code{hapFabia}. Segments have been
  generated by \code{split_sparse_matrix}.


}
\usage{

iterateSegments(startRun=1,endRun=640,shift=5000,segmentSize=10000,annotationFile=NULL,fileName="ALL.chr1.phase1_release_v2.20101123.snps_indels_svs",prefixPath="",sparseMatrixPostfix="_mat",annotPostfix="_annot.txt",samplesPostfix="_samples.txt",samples=0,lowerBP=0,upperBP=0.05,p=10,iter=40,quant=0.01,eps=1e-5,alpha=0.03,cyc=50,non_negative=1,write_file=0,norm=0,lap=100.0,IBDlength=50,Lt = 0.1,Zt = 0.2,thresCount=1e-5,minSNPsFactor=3/4,pMAF=0.035,haplotypes=TRUE)

}
\arguments{
 \item{startRun}{first segment.}
 \item{endRun}{last segment.}
 \item{shift}{distance between start of adjacent segments.}
 \item{segmentSize}{number of SNVs or SNPs in a segment.}
 \item{annotationFile}{file name of the annotation file.}
 \item{fileName}{passed to hapFabia: file name of the genotype matrix in sparse format.}
 \item{prefixPath}{passed to hapFabia: path to the genotype file.}
 \item{sparseMatrixPostfix}{passed to hapFabia: postfix string for the sparse matrix.}
 \item{annotPostfix}{passed to hapFabia: postfix string for the annotation file.}
 \item{samplesPostfix}{passed to hapFabia: postfix string for the file containing sample names.}
 \item{samples}{passed to hapFabia: vector of samples which should be included into the analysis; default = 0 (all samples).}
 \item{lowerBP}{passed to hapFabia: lower bound on minor allele frequencies (MAF); however at least two occurrences are required to remove private SNPs.}
 \item{upperBP}{passed to hapFabia: upper bound  on minor allele frequencies (MAF) to extract rare variants.}
 \item{p}{passed to hapFabia: number of biclusters per iteration.}
 \item{iter}{passed to hapFabia: number of iterations.}
 \item{quant}{passed to hapFabia: percentage of loadings L to remove in each iteration.}
 \item{eps}{passed to hapFabia: lower bound for variational parameter lapla; default 1e-5.}
 \item{alpha}{passed to hapFabia: sparseness of the loadings; default = 0.03.}
 \item{cyc}{passed to hapFabia: number of cycles per iterations; default 50.}
 \item{non_negative}{non-negative factors and loadings if non_negative = 1; default = 1 (yes).}
 \item{write_file}{passed to hapFabia: results are written to files (L in sparse format), default = 0 (not written).}
 \item{norm}{passed to hapFabia: data normalization; default 1 (no normalization).}
 \item{lap}{passed to hapFabia: minimal value of the variational parameter; default 100.0.}
 \item{IBDlength}{passed to hapFabia: typical IBD or haplotype cluster length in kbp.}
 \item{Lt}{passed to hapFabia: percentage of largest Ls to consider for IBD/haplotype cluster extraction.}
 \item{Zt}{passed to hapFabia: percentage of largest Zs to consider for IBD/haplotype cluster extraction.}
 \item{thresCount}{passed to hapFabia: p-value of random histogram hit; default 1e-5.}
 \item{minSNPsFactor}{passed to hapFabia: percentage of IBD/haplotype cluster overlap; default 3/4.}
 \item{pMAF}{passed to hapFabia: averaged and corrected (for non-uniform distributions) minor allele frequency.}
\item{haplotypes}{haplotypes = TRUE then phased genotypes meaning two chromosomes per individual otherwise unphased genotypes.}


}
\details{


  Implementation in \R. Reads annotation of the samples if available.
  Then calls \code{hapFabia}.
  Results are saved in EXCEL format and as \R
  binaries.

}
\seealso{
\code{\link{hapFabia}},
\code{\link{vcftoFABIA}},
\code{\link{split_sparse_matrix}},
\code{\link{iterateSegments}},
\code{\link{getIBDs}},
\code{\link{extractIBD}},
\code{\link{compareIBDres}},
\code{\link{makePipelineFile}},
\code{\link{mergeIBDres}},
\code{\link{identifyDuplicates}},
\code{\link{ibd2excel}},
\code{\link{plotRangeL}},
\code{\link{plotOneIBD}},
\code{\link{plotIBDs}},
\code{\link{snpPlot}},
\code{\link{analyzeIBDs}},
\code{\link{simulateHaplotypes}},
\code{\link{simulateForFabia}},
\code{\link{sim}},
\code{\link{topLZ}},
\code{\link{histL}},
\code{\link{plotL}},
\code{\link{hapRes}},
\code{\link{res}},
\code{\link{simu}},
\code{\link{mergedIBD}}
}
\author{Sepp Hochreiter}
\references{

S. Hochreiter et al.,
\sQuote{FABIA: Factor Analysis for Bicluster Acquisition},
Bioinformatics 26(12):1520-1527, 2010.

}
\keyword{models,multivariate,cluster}
\concept{genetics,haplotype,identity by descent,bicluster,next generation sequencing,genotype,single nucleotide polymorphism,single nucleotide variation,rare variants,rare SNPs, rare SNVs,rare haplotype clusters,short haplotype clusters}
